package com.blazinggames.musatic
{
	import com.blazinggames.musatic.assets.Assets;
	import com.blazinggames.musatic.assets.IScreenManager;
	
	import flash.geom.Point;
	import flash.media.SoundMixer;
	import flash.media.SoundTransform;
	
	import starling.display.Button;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.text.TextField;
	import starling.textures.Texture;
	
	public class CheatScreen extends Sprite
	{
		private var _screenManager:IScreenManager;
		private var _noteGrid:Array;
		private var _playButton:Button;
		private var _song:Song;
		private var _volume:Slider;
		private var _stepGrid:Array
		public function CheatScreen(controller:IScreenManager, song:Song)
		{
			super();
			var cntrRow:int, cntrCol:int;
			_song = song;
			_screenManager = controller;
			_noteGrid = new Array();
			_stepGrid = new Array();
			var note:NoteBox;
			
			var caption:TextField = new TextField(640,32, "Notes", "_sans", 25, 0xCCCC88, true);
			addChild(caption);
			caption = new TextField(640,32, "Steps", "_sans", 25, 0xCCCC88, true);
			caption.y = 296
			addChild(caption);
			
			for (cntrCol = 0; cntrCol < _song.length; ++cntrCol) {
				_noteGrid[cntrCol] = new Array();
				_stepGrid[cntrCol] = new Array();
				for (cntrRow = 0; cntrRow < 11; ++cntrRow) {
					note = new NoteBox(cntrRow, cntrCol);
					note.x = cntrCol * 8;
					note.y = cntrRow * 24 + 32;
//					note.addEventListener(NoteBox.NOTE_TOGGLED_EVENT, handleNoteTouch);
					addChild(note);
					_noteGrid[cntrCol][cntrRow] = note;
				}
				for (cntrRow = 0; cntrRow < 4; ++cntrRow) {
					note = new NoteBox(cntrRow, cntrCol);
					note.x = cntrCol * 8;
					note.y = cntrRow * 24 + 330;
//					note.addEventListener(NoteBox.NOTE_TOGGLED_EVENT, handleDirectionTouch);
					addChild(note);
					_stepGrid[cntrCol][cntrRow] = note;
				}
			}
			
			_playButton = new Button(Assets.getSpriteSheet().getTexture("MiniPlayButton0000"));
			_playButton.x = 540;
			_playButton.y = 440;
			_playButton.addEventListener(Event.TRIGGERED, handlePlaySong);
			addChild(_playButton);
			
			_volume = new Slider(Texture.fromColor(300,20,0xFF444444), Texture.fromColor(8,20), new Point(0,0), new Point(292,0));
			addChild(_volume);
			_volume.x = 100;
			_volume.y = 440;
			_volume.addEventListener(Slider.VALUE_CHANGED_EVENT, hanldeSoundVolume);
			
			addEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			addEventListener(Event.REMOVED_FROM_STAGE, handleRemovedFromStage);
		}
		
		private function handleAddedToStage(e:Event):void
		{
			var cntrCol:int, cntrRow:int;

			_volume.value = SoundMixer.soundTransform.volume;
			setGridFromSong();
			for (cntrCol = 0; cntrCol < _song.length; ++cntrCol) {
				for (cntrRow = 0; cntrRow < 11; ++cntrRow) {
					_noteGrid[cntrCol][cntrRow].addEventListener(NoteBox.NOTE_TOGGLED_EVENT, handleNoteTouch);
				}
				for (cntrRow = 0; cntrRow < 4; ++cntrRow) {
					_stepGrid[cntrCol][cntrRow].addEventListener(NoteBox.NOTE_TOGGLED_EVENT, handleDirectionTouch);
				}
			}
		}
		
		private function handleRemovedFromStage(e:Event):void
		{
			var cntrCol:int, cntrRow:int;
			
			for (cntrCol = 0; cntrCol < _song.length; ++cntrCol) {
				for (cntrRow = 0; cntrRow < 11; ++cntrRow) {
					_noteGrid[cntrCol][cntrRow].removeEventListener(NoteBox.NOTE_TOGGLED_EVENT, handleNoteTouch);
				}
				for (cntrRow = 0; cntrRow < 4; ++cntrRow) {
					_stepGrid[cntrCol][cntrRow].removeEventListener(NoteBox.NOTE_TOGGLED_EVENT, handleDirectionTouch);
				}
			}
		}

		private function hanldeSoundVolume():void
		{
			trace ("Old volume = " + SoundMixer.soundTransform.volume);
			SoundMixer.soundTransform = new SoundTransform(_volume.value);
			trace("Volume should be " + _volume.value);
			trace ("New volume = " + SoundMixer.soundTransform.volume);
			_song.playChord(42);
		}
		
		private function setGridFromSong():void
		{
			var cntrRow:int, cntrCol:int, chord:int;
			for (cntrCol = 0; cntrCol < _song.length; ++cntrCol) {
				chord = _song.getChord(cntrCol);
				for (cntrRow = 0; cntrRow < 11; ++cntrRow) {
					_noteGrid[cntrCol][cntrRow].noteState((chord & Song.NOTE_MASKS[cntrRow]) > 0)
				}
				chord = _song.getStep(cntrCol);
				for (cntrRow = 0; cntrRow < 4; ++cntrRow) {
					_stepGrid[cntrCol][cntrRow].noteState((chord & Song.NOTE_MASKS[cntrRow]) > 0)
				}
			}
		}
		private function handleNoteTouch(e:Event):void
		{
			var note:NoteBox = NoteBox(e.target);
			note.noteState(!note.isNote());
			var mask:int = Song.NOTE_MASKS[note.row];
			if (note.isNote())
				_song.setChord(note.column, _song.getChord(note.column) | mask);
			else
				_song.setChord(note.column, (0xFFFF ^ mask) & _song.getChord(note.column));
			SoundMixer.stopAll();
			_song.playChord(_song.getChord(note.column));
		}
		
		private function handleDirectionTouch(e:Event):void
		{
			var note:NoteBox = NoteBox(e.target);
			note.noteState(!note.isNote());
			var mask:int = Song.NOTE_MASKS[note.row];
			if (note.isNote())
				_song.setStep(note.column, _song.getStep(note.column) | mask);
			else
				_song.setStep(note.column, (0xFFFF ^ mask) & _song.getStep(note.column));
		}
		
		private function handlePlaySong(e:Event):void
		{
			// TODO Auto Generated method stub
//			_song.playSong(stage);
			_screenManager.changeScreen("Play");
		}
		
	}
}