package com.blazinggames.musatic
{
	import com.blazinggames.musatic.assets.IScreenManager;
	
	import flash.ui.Keyboard;
	import flash.utils.getTimer;
	
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.display.Sprite;
	import starling.display.Stage;
	import starling.events.Event;
	import starling.events.KeyboardEvent;
	import starling.text.TextField;
	
	public class CreateScreen extends Sprite
	{
		private static const STATE_STAGE1_PROMPT:int = 0;
		private static const STATE_STAGE1:int = 1;
		private static const STATE_STAGE2_PROMPT:int = 2;
		private static const STATE_STAGE2:int = 3;
		private static const STATE_DONE:int = 4;
		
		private static const PHASE_1_PROMPT_TEXT:String = "Phase 1:\n Use number keys to create song.";
		private static const PHASE_2_PROMPT_TEXT:String = "Phase 2:\n Use cursor keys to create dance.";
		private static const DONE_PROMPT:String = "Hit any key to play your game";
		
// ***** VARIABLES *****
		
		private var _screenManager:IScreenManager;
		private var _song:Song;
		private var _stage:Stage;
		private var _message:TextField;
		private var _state:int;
		private var _nextChordTime:int;
		private var _currentColumn:int;
		private var _targets:Array;
		
		public function CreateScreen(controller:IScreenManager, song:Song)
		{
			super();
			_song = song;
			_screenManager = controller;
			_targets = new Array();

			var target:Arrow;
			target = new Arrow(Arrow.LEFT);
			target.x = 224;
			target.y = 208;
			target.alpha = 0;
			addChild(target);
			_targets[0] = target;
			
			target = new Arrow(Arrow.UP);
			target.x = 288;
			target.y = 144;
			target.alpha = 0;
			addChild(target);
			_targets[1] = target;
			
			target = new Arrow(Arrow.RIGHT);
			target.x = 352;
			target.y = 208;
			target.alpha = 0;
			addChild(target);
			_targets[2] = target;
			
			target = new Arrow(Arrow.DOWN);
			target.x = 288;
			target.y = 272;
			target.alpha = 0;
			addChild(target);
			_targets[3] = target;

			addEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			_message = new TextField(640,480, PHASE_1_PROMPT_TEXT, "_sans", 32,0xCCCC00);
			addChild(_message);
		}
		
		private function handleAddedToStage(e:Event):void
		{
			_state = 0;
			_song.clearSong();
			_message.text = PHASE_1_PROMPT_TEXT;
			removeEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			addEventListener(Event.REMOVED_FROM_STAGE, handleRemovedFromStage);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			_stage = stage;
			addEventListener(Event.ENTER_FRAME, handleEnterFrame);
		}
		
		private function handleRemovedFromStage(e:Event):void
		{
			addEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			removeEventListener(Event.REMOVED_FROM_STAGE, handleRemovedFromStage);
			_stage.removeEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			removeEventListener(Event.ENTER_FRAME, handleEnterFrame);
		}
		
		private function handleEnterFrame(e:Event):void
		{
			if ((_state == STATE_STAGE1) || (_state == STATE_STAGE2)){
				if (_nextChordTime <= getTimer()) {
					_nextChordTime += 250;
					++_currentColumn;
					_message.text = "" + Math.ceil(20 - _currentColumn / 4);
					if (_currentColumn >= _song.length) {
						if (_state == STATE_STAGE1) {
							_state = STATE_STAGE2_PROMPT;
							_message.text = PHASE_2_PROMPT_TEXT;
						} else {
							_state = STATE_DONE;
							_message.text = DONE_PROMPT;							
						}
					}
				}
			}
		}

		private function handleKeyDown(e:KeyboardEvent):void
		{
			var id:int = Math.min(10, Math.max(0, e.keyCode - Keyboard.NUMBER_1));
			switch (_state) {
				case STATE_STAGE1_PROMPT:
					_nextChordTime = getTimer();
					_currentColumn = 0;
					_state = STATE_STAGE1;
					break;
				case STATE_STAGE1:
					_song.setChord(_currentColumn, _song.getChord(_currentColumn) | Song.NOTE_MASKS[id]);
					_song.playChord(Song.NOTE_MASKS[id]);
					break;
				case STATE_STAGE2_PROMPT:
					_nextChordTime = getTimer();
					_currentColumn = 0;
					_state = STATE_STAGE2;
					_song.playSong(stage);
					break;
				case STATE_STAGE2:
					id = e.keyCode - 37;
					if ((id >= 0) && (id < 4)) {
						_song.setStep(_currentColumn, _song.getStep(_currentColumn) | Song.NOTE_MASKS[id]);
						_targets[id].alpha = 1;
						var tween:Tween = new Tween(_targets[id],1);
						tween.fadeTo(0);
						Starling.juggler.add(tween);
					}
					break;
				case STATE_DONE:
					_screenManager.changeScreen("Play");
					break;
			}
		}

	}
}