package com.blazinggames.musatic
{
	import com.blazinggames.musatic.assets.IScreenManager;
	
	import flash.media.SoundMixer;
	import flash.utils.ByteArray;
	
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.display.Sprite;
	import starling.display.Stage;
	import starling.events.Event;
	import starling.events.KeyboardEvent;
	
	public class PlayScreen extends Sprite
	{
		private var _screenManager:IScreenManager;
		private var _song:Song;
		private var _soundBarBackdrop:Array;
		private var _soundBytes:ByteArray;
		private var _lastChord:int;
		private var _arrowPool:Array;
		private var _arrows:Array;
		private var _results:Array;
		private var _resultsPool:Array;
		private var _stage:Stage;
		private var _targets:Array;
		
		public function PlayScreen(controller:IScreenManager, song:Song)
		{
			super();
			_song = song;
			_screenManager = controller;
			_soundBarBackdrop = new Array();
			for (var cntr:int = 0; cntr < song.length; ++cntr)
			{
				_soundBarBackdrop[cntr] = new NoteBox(cntr, 0);
				_soundBarBackdrop[cntr].x = 8 * cntr;
				_soundBarBackdrop[cntr].y = 240;
				_soundBarBackdrop[cntr].noteState(true);
				addChild(_soundBarBackdrop[cntr]);
			}
			_soundBytes = new ByteArray();
			addEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			
			_targets = new Array();
			var target:Arrow;
			target = new Arrow(Arrow.LEFT);
			target.x = 224;
			target.y = 208;
			addChild(target);
			_targets[0] = target;

			target = new Arrow(Arrow.UP);
			target.x = 288;
			target.y = 144;
			addChild(target);
			_targets[1] = target;

			target = new Arrow(Arrow.RIGHT);
			target.x = 352;
			target.y = 208;
			addChild(target);
			_targets[2] = target;

			target = new Arrow(Arrow.DOWN);
			target.x = 288;
			target.y = 272;
			addChild(target);
			_targets[3] = target;

			_arrowPool = new Array();
			_arrows = new Array();
			_results = new Array();
			_resultsPool = new Array();
		}
		
		private function handleAddedToStage(e:Event):void
		{
			addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			_lastChord = 0;
			_song.playSong(stage);
			_song.addEventListener(Song.SONG_FINISHED_EVENT, handleSongFinished);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			_stage = stage;
			
		}
		
		private function findDistanceSquared(a:Sprite, b:Sprite):Number
		{
			var deltaX:Number = b.x - a.x;
			var deltaY:Number = b.y - a.y;
			return (deltaX * deltaX + deltaY * deltaY);
		}
		
		private function handleKeyDown(e:KeyboardEvent):void
		{
			var dir:int = e.keyCode - 37;
			if ((dir < 0) || (dir > 3))
				return;
			var cntr:int, arrowIndex:int = -1;
			var dist:Number;
			
			for (cntr = 0; cntr < _arrows.length; ++cntr)
				if (_arrows[cntr].direction == dir) {
					dist = findDistanceSquared(_arrows[cntr], _targets[dir]);
					trace (" arrow distance " + dist);
					if (dist < 4096) {
						arrowIndex = cntr;
						var rating:int = 3;
						if (dist < 64)
							rating = 0;
						else if (dist < 256)
							rating = 1;
						else if (dist < 2304)
							rating = 2;
						var result:Results;
						if (_resultsPool.length > 0) {
							result = _resultsPool.pop();
							result.result = rating;
							result.alpha = 1;
						} else
							result = new Results(rating);
						result.tween = new Tween(result, 2);
						result.tween.fadeTo(0);
						result.x = _arrows[cntr].x;
						result.y = _arrows[cntr].y;
						addChild(result);
						_results.push( result);
						Starling.juggler.add(result.tween);
					}
				}
			if (arrowIndex >= 0) 
				_arrows[arrowIndex].visible = false;
		}
		
		private function handleSongFinished():void
		{
			removeEventListener(Event.ENTER_FRAME, handleEnterFrame);
			stage.removeEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			_screenManager.changeScreen("Title");
		}
		
		private function createArrow(dir:int):void
		{
			var arrow:Arrow;
			if (_arrowPool.length > 0) {
				arrow = _arrowPool.pop();
				arrow.direction = dir;
				arrow.visible = true;
			} else
				arrow = new Arrow(dir);
			arrow.tween = new Tween(arrow, 2);
			switch (dir) {
				case Arrow.LEFT:
					arrow.x = 0;
					arrow.y = 208;
					arrow.tween.animate("x", 224);
					break;
				case Arrow.UP:
					arrow.x = 288;
					arrow.y = -32;
					arrow.tween.animate("y", 144);
					break;
				case Arrow.RIGHT:
					arrow.x = 579;
					arrow.y = 208;
					arrow.tween.animate("x", 352);
					break;
				case Arrow.DOWN:
					arrow.x = 288;
					arrow.y = 450;
					arrow.tween.animate("y", 272);
					break;
			}
			addChild(arrow);
			_arrows.push(arrow);
			Starling.juggler.add(arrow.tween);
		}
		
		private function handleEnterFrame(e:Event):void
		{
			SoundMixer.computeSpectrum(_soundBytes, false, 0)
			var cntr:int;
			
			for (cntr = 0; cntr < _song.length; ++cntr)
			{
				_soundBarBackdrop[cntr].y = _soundBytes.readFloat() * 240 + 240 ;
			}
			
			if (_lastChord != _song.songPosition) {
				_lastChord = _song.songPosition;
				var col:int = _lastChord + 8;
				if (col < _song.length) {
					var stepSet:int = _song.getStep(col);
					if (stepSet & 1)
						createArrow(Arrow.LEFT);
					if (stepSet & 2)
						createArrow(Arrow.UP);
					if (stepSet & 4)
						createArrow(Arrow.RIGHT);
					if (stepSet & 8)
						createArrow(Arrow.DOWN);		
				}
			}
			// clean up dead arrows
			var toRemove:int = -1;
			for (cntr = 0; cntr < _arrows.length; ++cntr)
				if (_arrows[cntr].tween.isComplete)
					toRemove = cntr;
			if (toRemove >= 0) {
				var arrow:Arrow = _arrows[toRemove];
				_arrows.splice(toRemove,1);
				_arrowPool.push(arrow);
				removeChild(arrow);
				trace("Removing Arrow. Pool size now: " + _arrowPool.length + " on Screen arrows " + _arrows.length);
			}
			toRemove = -1;
			for (cntr = 0; cntr < _results.length; ++cntr)
				if (_results[cntr].tween.isComplete)
					toRemove = cntr;
			if (toRemove >= 0) {
				var result:Results = _results[toRemove];
				_results.splice(toRemove,1);
				_resultsPool.push(result);
				removeChild(result);
				trace("Removing Result. Pool size now: " + _resultsPool.length + " on Screen results " + _results.length);
			}
		}
				
	}
}