package com.blazinggames.musatic
{
	import flash.utils.getTimer;
	
	import starling.display.Stage;
	import starling.events.Event;
	import starling.events.EventDispatcher;

	public class Song extends EventDispatcher
	{
// ***** CONSTANTS *****
		
		public static const SONG_FINISHED_EVENT:String = "Song Finished";
		public static const NOTE_MASKS:Array = new Array (
			1,2,4,8,16,32,64,128,256, 0x200, 0x400, 0x800, 0x1000, 0x2000, 0x4000);

// ***** VARIABLES *****
		
		private var _sounds:Array;
		private var _chords:Array;
		private var _steps:Array;
		
		private var _isPlaying:Boolean;
		private var _playColumn:int;
		private var _nextChordTime:int;
		private var _stage:Stage;
		
// ***** CONSTRUCTOR *****
		
		public function Song(soundSet:Array, length:int = 80)
		{
			super();
			_sounds = soundSet;
			_chords = new Array(length);
			_steps = new Array(length);
		}
		
		public function changeSoundSet(sounds:Array):void
		{
			_sounds = sounds;
		}
		
		public function playChord(notes:int):void
		{
			if (_sounds == null) return;
			var len:int = _sounds.length;
			for (var cntr:int = 0; cntr < len; ++cntr) {
				if ((notes & NOTE_MASKS[cntr]) > 0)
					_sounds[cntr].play();
			}
		}
		
		public function playSong(stage:Stage):void
		{
			_stage = stage;
			stage.addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			_playColumn = 0;
			_nextChordTime = getTimer();
			_isPlaying = true;			
		}
		
		public function stopSong():void
		{
			_isPlaying = false;
			_stage.removeEventListener(Event.ENTER_FRAME, handleEnterFrame);			
		}
		
		private function handleEnterFrame(e:Event):void
		{
			if (_isPlaying) {
				if (_nextChordTime <= getTimer()) {
					_nextChordTime += 250;
					playChord(_chords[_playColumn]);
					++_playColumn;
					if (_playColumn >= _chords.length) {
						_isPlaying = false;
						_stage.removeEventListener(Event.ENTER_FRAME, handleEnterFrame);
						dispatchEvent(new Event(SONG_FINISHED_EVENT));
					}
				}
			}
		}
		
		public function getChord(place:int):int
		{
			return _chords[place];
		}
		
		public function setChord(place:int, chord:int):void
		{
			_chords[place] = chord;
		}
		
		public function getStep(place:int):int
		{
			return _steps[place];
		}
		
		public function setStep(place:int, step:int):void
		{
			_steps[place] = step;
		}

		public function clearSong():void
		{
			var cntr:int;
			var len:int = _chords.length;
			
			for (cntr = 0; cntr < len; ++cntr) {
				_chords[cntr] = 0;
				_steps[cntr] = 0;
			}
		}

		public function buildTestSong():void
		{
			var cntr:int;
			var len:int = _chords.length;
			
			for (cntr = 0; cntr < len; ++cntr)
				_chords[cntr] = 0;
			for (cntr = 0; cntr < 11; ++cntr)
				_chords[cntr * 2] = NOTE_MASKS[cntr];
			for (cntr = 0; cntr < 4; ++cntr)
				_steps[12 + cntr * 4] = NOTE_MASKS[cntr];
		}

		public function get length():int
		{
			return _chords.length;
		}

		public function get songPosition():int
		{
			return _playColumn;
		}


	}
}