package com.blazinggames.musatic
{
	import com.blazinggames.musatic.assets.Assets;
	import com.blazinggames.musatic.assets.IScreenManager;
	
	import flash.geom.Point;
	import flash.media.SoundMixer;
	import flash.media.SoundTransform;
	
	import starling.display.Button;
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.text.TextField;
	import starling.textures.Texture;
	import starling.textures.TextureAtlas;
	
	public class TitleScreen extends Sprite
	{
		private var _screenManager:IScreenManager;
		private var _create:Button;
		private var _play:Button;
		private var _edit:Button;
		private var _about:Button;
		private var _song:Song;
		private var _volume:Slider;
		
		public function TitleScreen(controller:IScreenManager)
		{
			super();
			_screenManager = controller;
			_song = new Song(Assets.getSoundSet());
			_song.buildTestSong();
			var spriteSheet:TextureAtlas = Assets.getSpriteSheet();
			var title:Image = new Image(spriteSheet.getTexture("Title0000"));
			title.x = 70;
			addChild(title);
			
			_create = new Button(spriteSheet.getTexture("CreateButton0000"));
			_create.x = 160;
			_create.y = 150;
			_create.addEventListener(Event.TRIGGERED, handleCreateButton);
			addChild(_create);
			
			_play = new Button(spriteSheet.getTexture("PlayButton0000"));
			_play.x = 160;
			_play.y = 210;
			_play.addEventListener(Event.TRIGGERED, handlePlayButton);
			addChild(_play);

			_edit = new Button(spriteSheet.getTexture("EditButton0000"));
			_edit.x = 160;
			_edit.y = 270;
			_edit.addEventListener(Event.TRIGGERED, handleEditButton);
			addChild(_edit);
						
			_about = new Button(spriteSheet.getTexture("AboutButton0000"));
			_about.x = 160;
			_about.y = 330;
			_about.addEventListener(Event.TRIGGERED, handleAboutButton);
			addChild(_about);

			var volIcon:Image = new Image(spriteSheet.getTexture("VolumeIcon0000"));
			volIcon.scaleX = .25;
			volIcon.scaleY = .25;
			volIcon.y = 410;
			volIcon.x = 100;
			addChild(volIcon);
			
			_volume = new Slider(Texture.fromColor(300,20,0xFF444444), Texture.fromColor(8,20), new Point(0,0), new Point(292,0));
			addChild(_volume);
			_volume.x = 160;
			_volume.y = 420;
			_volume.addEventListener(Slider.VALUE_CHANGED_EVENT, hanldeSoundVolume);

			var copyright:TextField = new TextField(640,20,"Copyright ©2012 Blazing Games Inc. Some Rights Reserved.", "_sans", 12, 0xFFFF00);
			copyright.y = 460;
			addChild(copyright);
			
			addEventListener(Event.ADDED_TO_STAGE, handleAddedToStage);
			addEventListener(Event.REMOVED_FROM_STAGE, handleRemovedFromStage);
		}

		private function hanldeSoundVolume():void
		{
			SoundMixer.soundTransform = new SoundTransform(_volume.value);
			_song.playChord(42);
		}
		
		private function handleAddedToStage(e:Event):void
		{
			_song.playSong(stage);
			_volume.value = SoundMixer.soundTransform.volume;
		}

		private function handleRemovedFromStage(e:Event):void
		{
			_song.stopSong();
		}
		
		private function handleCreateButton(e:Event):void
		{
			_screenManager.changeScreen("Create");
		}

		private function handlePlayButton(e:Event):void
		{
			_screenManager.changeScreen("Play");
		}
		
		private function handleEditButton(e:Event):void
		{
			_screenManager.changeScreen("Editor");
		}
		
		private function handleAboutButton(e:Event):void
		{
			_screenManager.changeScreen("About");
		}
	}
}